//
//  ParameterParsingError.cs
//
//  Author:
//       Jarl Gullberg <jarl.gullberg@gmail.com>
//
//  Copyright (c) 2017 Jarl Gullberg
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using JetBrains.Annotations;
using Remora.Commands.Signatures;
using Remora.Results;

namespace Remora.Commands.Results
{
    /// <summary>
    /// Represents a failure to parse a bound parameter.
    /// </summary>
    [PublicAPI]
    public record ParameterParsingError : ResultError
    {
        /// <summary>
        /// Gets the parameter that failed to parse.
        /// </summary>
        public BoundParameterShape Parameter { get; }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParameterParsingError"/> class.
        /// </summary>
        /// <param name="parameter">The parameter that failed to parse.</param>
        public ParameterParsingError(BoundParameterShape parameter)
            : base($"Failed to parse the value of {parameter.ParameterShape.HintName}.")
        {
            this.Parameter = parameter;
        }
    }
}
